/***************************************************************************
 *   Copyright (C) 2005 by Christophe GONZALES and Pierre-Henri WUILLEMIN  *
 *   {prenom.nom}_at_lip6.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/** @file
 * @brief Base Class for all the algorithms producing a junction given a set
 * of cliques/subcliques resulting from a triangulation
 *
 * @author Christophe GONZALES and Pierre-Henri WUILLEMIN
 */

#include <agrum/graphs/triangulations/junctionTreeStrategies/junctionTreeStrategy.h>

namespace gum {

  // default constructor
  JunctionTreeStrategy::JunctionTreeStrategy() {
    // for debugging purposes
    GUM_CONSTRUCTOR( JunctionTreeStrategy );
  }

  // copy constructor
  JunctionTreeStrategy::JunctionTreeStrategy( const JunctionTreeStrategy& from )
      : _triangulation( from._triangulation ) {
    // for debugging purposes
    GUM_CONS_CPY( JunctionTreeStrategy );
  }

  // move constructor
  JunctionTreeStrategy::JunctionTreeStrategy( JunctionTreeStrategy&& from )
      : _triangulation( from._triangulation ) {
    // for debugging purposes
    GUM_CONS_MOV( JunctionTreeStrategy );
  }

  // destructor
  JunctionTreeStrategy::~JunctionTreeStrategy() {
    // for debugging purposes
    GUM_DESTRUCTOR( JunctionTreeStrategy );
  }

  /** @brief assigns a new triangulation to the junction tree strategy
   * during a move construction */
  void
  JunctionTreeStrategy::moveTriangulation( StaticTriangulation* triangulation ) {
    _triangulation = triangulation;
  }

} /* namespace gum */
